@@ -34,9 +34,8 @@ rwlock_read_lock(pthread_rwlock_t *lock)
 	 * pthread_rwlock_rdlock() failing like this is akin to `if` failing;
 	 * we're screwed badly, so let's just pull the trigger.
 	 */
-	pr_op_err("pthread_rwlock_rdlock() returned error code %d. This is too critical for a graceful recovery; I must die now.",
+	pr_crit("pthread_rwlock_rdlock() returned error code %d. This is too critical for a graceful recovery; I must die now.",
 	    error);
-	exit(error);
 }
 
 void
@@ -49,11 +48,9 @@ rwlock_write_lock(pthread_rwlock_t *lock)
 	 * Both of them indicate serious programming errors.
 	 */
 	error = pthread_rwlock_wrlock(lock);
-	if (error) {
-		pr_op_err("pthread_rwlock_wrlock() returned error code %d. This is too critical for a graceful recovery; I must die now.",
+	if (error)
+		pr_crit("pthread_rwlock_wrlock() returned error code %d. This is too critical for a graceful recovery; I must die now.",
 		    error);
-		exit(error);
-	}
 }
 
 void
@@ -66,11 +63,9 @@ rwlock_unlock(pthread_rwlock_t *lock)
 	 * Both of them indicate serious programming errors.
 	 */
 	error = pthread_rwlock_unlock(lock);
-	if (error) {
-		pr_op_err("pthread_rwlock_unlock() returned error code %d. This is too critical for a graceful recovery; I must die now.",
+	if (error)
+		pr_crit("pthread_rwlock_unlock() returned error code %d. This is too critical for a graceful recovery; I must die now.",
 		    error);
-		exit(error);
-	}
 }
 
 void
@@ -205,13 +200,15 @@ valid_file_or_dir(char const *location, bool check_file, bool check_dir,
 	result = false;
 	file = fopen(location, "rb");
 	if (file == NULL) {
-		cb(errno, "Could not open location '%s'",
-		    location);
+		if (cb != NULL)
+			cb(errno, "Could not open location '%s'",
+			    location);
 		return false;
 	}
 
 	if (fstat(fileno(file), &attr) == -1) {
-		cb(errno, "fstat(%s) failed", location);
+		if (cb != NULL)
+			cb(errno, "fstat(%s) failed", location);
 		goto end;
 	}
 
@@ -226,7 +223,8 @@ valid_file_or_dir(char const *location, bool check_file, bool check_dir,
 
 end:
 	if (fclose(file) == -1)
-		cb(errno, "fclose() failed");
+		if (cb != NULL)
+			cb(errno, "fclose() failed");
 	return result;
 }
 
@@ -417,3 +415,62 @@ get_current_time(time_t *result)
 	*result = now;
 	return 0;
 }
+
+/*
+ * Maps an absolute @uri that begins with @uri_prefix (either 'rsync://' or
+ * 'https://') to a local URI. If a @workspace is set, append such location
+ * to the local-repository location (this workspace is used at https URIs).
+ *
+ * @result is allocated with the local URI.
+ *
+ * Returns 0 on success, otherwise an error code.
+ */
+int
+map_uri_to_local(char const *uri, char const *uri_prefix, char const *workspace,
+    char **result)
+{
+	char const *repository;
+	char *local;
+	size_t repository_len;
+	size_t uri_prefix_len;
+	size_t uri_len;
+	size_t workspace_len;
+	size_t extra_slash;
+	size_t offset;
+
+	repository = config_get_local_repository();
+	repository_len = strlen(repository);
+	uri_prefix_len = strlen(uri_prefix);
+	uri_len = strlen(uri);
+
+	uri += uri_prefix_len;
+	uri_len -= uri_prefix_len;
+	extra_slash = (repository[repository_len - 1] == '/') ? 0 : 1;
+
+	workspace_len = 0;
+	if (workspace != NULL)
+		workspace_len = strlen(workspace);
+
+	local = malloc(repository_len + extra_slash + workspace_len + uri_len +
+	    1);
+	if (local == NULL)
+		return pr_enomem();
+
+	offset = 0;
+	strcpy(local + offset, repository);
+	offset += repository_len;
+	if (extra_slash) {
+		strcpy(local + offset, "/");
+		offset += extra_slash;
+	}
+	if (workspace_len > 0) {
+		strcpy(local + offset, workspace);
+		offset += workspace_len;
+	}
+	strncpy(local + offset, uri, uri_len);
+	offset += uri_len;
+	local[offset] = '\0';
+
+	*result = local;
+	return 0;
+}
