@@ -5,14 +5,13 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <ftw.h>
-#include <pthread.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include "common.h"
+#include "internal_pool.h"
 #include "log.h"
 #include "random.h"
-#include "uri.h"
 
 #define MAX_FD_ALLOWED 20
 
@@ -81,9 +80,8 @@ remove_from_root(void *arg)
 	dirs_arr = root_arg->arr;
 	len = root_arg->arr_set;
 
-	/* Release received arg, and detach thread */
+	/* Release received arg */
 	free(root_arg);
-	pthread_detach(pthread_self());
 
 	for (i = 0; i < len; i++) {
 		error = nftw(dirs_arr[i], traverse, MAX_FD_ALLOWED,
@@ -112,23 +110,18 @@ remove_from_root(void *arg)
  * - '= 0' no error
  */
 static int
-get_local_path(char const *rcvd, char **result)
+get_local_path(char const *rcvd, char const *workspace, char **result)
 {
 	struct stat attr;
-	struct rpki_uri *uri;
 	char *tmp, *local_path;
 	size_t tmp_size;
 	int error;
 
-	error = uri_create_mixed_str(&uri, rcvd, strlen(rcvd));
+	/* Currently, only rsync URIs are utilized */
+	local_path = NULL;
+	error = map_uri_to_local(rcvd, "rsync://", workspace, &local_path);
 	if (error)
-		return error != -ENOMEM ? EINVAL : error;
-
-	local_path = strdup(uri_get_local(uri));
-	if (local_path == NULL) {
-		error = pr_enomem();
-		goto release_uri;
-	}
+		return error;
 
 	error = stat(local_path, &attr);
 	if (error) {
@@ -161,14 +154,11 @@ get_local_path(char const *rcvd, char **result)
 	tmp[tmp_size] = '\0';
 
 	free(local_path);
-	uri_refput(uri);
 
 	*result = tmp;
 	return 0;
 release_local:
 	free(local_path);
-release_uri:
-	uri_refput(uri);
 	return error;
 }
 
@@ -212,7 +202,7 @@ rename_local_path(char const *rcvd, char **result)
 }
 
 static int
-rename_all_roots(struct rem_dirs *rem_dirs, char **src)
+rename_all_roots(struct rem_dirs *rem_dirs, char **src, char const *workspace)
 {
 	char *local_path, *delete_path;
 	size_t i;
@@ -221,7 +211,7 @@ rename_all_roots(struct rem_dirs *rem_dirs, char **src)
 	for (i = 0; i < rem_dirs->arr_len; i++) {
 		local_path = NULL;
 		error = get_local_path(src[(rem_dirs->arr_len - 1) - i],
-		    &local_path);
+		    workspace, &local_path);
 		if (error < 0)
 			return error;
 		if (error > 0)
@@ -274,7 +264,8 @@ rem_dirs_destroy(struct rem_dirs *rem_dirs)
 }
 
 /*
- * Remove the files listed at @roots array of @roots_len size.
+ * Remove the files listed at @roots array of @roots_len size. The local files
+ * will be searched at the specified HTTP local @workspace.
  * 
  * The daemon will be as quiet as possible, since most of its job is done
  * asynchronously. Also, it works on the best possible effort; some errors are
@@ -282,9 +273,8 @@ rem_dirs_destroy(struct rem_dirs *rem_dirs)
  * considers the relations (parent-child) at dirs.
  */
 int
-delete_dir_daemon_start(char **roots, size_t roots_len)
+delete_dir_daemon_start(char **roots, size_t roots_len, char const *workspace)
 {
-	pthread_t thread;
 	struct rem_dirs *arg;
 	int error;
 
@@ -293,18 +283,17 @@ delete_dir_daemon_start(char **roots, size_t roots_len)
 	if (error)
 		return error;
 
-	error = rename_all_roots(arg, roots);
+	error = rename_all_roots(arg, roots, workspace);
 	if (error) {
 		rem_dirs_destroy(arg);
 		return error;
 	}
 
-	/* Thread arg is released at thread before being detached */
-	errno = pthread_create(&thread, NULL, remove_from_root, (void *) arg);
-	if (errno) {
+	/* Thread arg is released at thread */
+	error = internal_pool_push(remove_from_root, (void *) arg);
+	if (error) {
 		rem_dirs_destroy(arg);
-		return pr_op_errno(errno,
-		    "Could not spawn the delete dir daemon thread");
+		return error;
 	}
 
 	return 0;
