@@ -9,8 +9,9 @@
 int http_init(void);
 void http_cleanup(void);
 
-typedef size_t (http_write_cb)(unsigned char *, size_t, size_t, void *);
-int http_download_file(struct rpki_uri *, http_write_cb, bool);
-int http_download_file_with_ims(struct rpki_uri *, http_write_cb, long, bool);
+int http_download_file(struct rpki_uri *, bool);
+int http_download_file_with_ims(struct rpki_uri *, long, bool);
+
+int http_direct_download(char const *, char const *);
 
 #endif /* SRC_HTTP_HTTP_H_ */
