@@ -1,9 +1,10 @@
 #ifndef SRC_OBJECT_MANIFEST_H_
 #define SRC_OBJECT_MANIFEST_H_
 
+#include <stdbool.h>
 #include "uri.h"
 #include "rpp.h"
 
-int handle_manifest(struct rpki_uri *, struct rpp **);
+int handle_manifest(struct rpki_uri *, bool, struct rpp **);
 
 #endif /* SRC_OBJECT_MANIFEST_H_ */
