@@ -95,31 +95,16 @@ rem_mft_from_list(struct visited_uris *visited_uris, char const *uri)
 	return visited_uris_remove(visited_uris, uri);
 }
 
-static size_t
-write_local(unsigned char *content, size_t size, size_t nmemb, void *arg)
-{
-	FILE *fd = arg;
-	size_t read = size * nmemb;
-	size_t written;
-
-	written = fwrite(content, size, nmemb, fd);
-	if (written != nmemb)
-		return -EINVAL;
-
-	return read;
-}
-
 static int
 download_file(struct rpki_uri *uri, long last_update, bool log_operation)
 {
 	int error;
 
 	if (last_update > 0)
-		error = http_download_file_with_ims(uri, write_local,
-		    last_update, log_operation);
-	else
-		error = http_download_file(uri, write_local,
+		error = http_download_file_with_ims(uri, last_update,
 		    log_operation);
+	else
+		error = http_download_file(uri, log_operation);
 
 	/*
 	 * Since distinct files can be downloaded (notification, snapshot,
@@ -523,7 +508,7 @@ parse_publish(xmlTextReaderPtr reader, bool parse_hash, bool hash_required,
 	uri = NULL;
 	if (tmp->doc_data.hash_len > 0) {
 		/* Get the current file from the uri */
-		error = uri_create_rsync_str(&uri, tmp->doc_data.uri,
+		error = uri_create_rsync_str_rrdp(&uri, tmp->doc_data.uri,
 		    strlen(tmp->doc_data.uri));
 		if (error)
 			goto release_base64;
@@ -565,7 +550,7 @@ parse_withdraw(xmlTextReaderPtr reader, struct withdraw **withdraw)
 		goto release_tmp;
 
 	/* rfc8181#section-2.2, get the file from the uri */
-	error = uri_create_rsync_str(&uri, tmp->doc_data.uri,
+	error = uri_create_rsync_str_rrdp(&uri, tmp->doc_data.uri,
 	    strlen(tmp->doc_data.uri));
 	if (error)
 		goto release_tmp;
@@ -596,7 +581,7 @@ write_from_uri(char const *location, unsigned char *content, size_t content_len,
 	int error;
 
 	/* rfc8181#section-2.2 must be an rsync URI */
-	error = uri_create_rsync_str(&uri, location, strlen(location));
+	error = uri_create_rsync_str_rrdp(&uri, location, strlen(location));
 	if (error)
 		return error;
 
@@ -655,7 +640,7 @@ __delete_from_uri(char const *location, struct visited_uris *visited_uris)
 	int error;
 
 	/* rfc8181#section-2.2 must be an rsync URI */
-	error = uri_create_rsync_str(&uri, location, strlen(location));
+	error = uri_create_rsync_str_rrdp(&uri, location, strlen(location));
 	if (error)
 		return error;
 
@@ -1032,7 +1017,7 @@ process_delta(struct delta_head *delta_head, void *arg)
 	head_data = &delta_head->doc_data;
 
 	pr_val_debug("Processing delta '%s'.", delta_head->doc_data.uri);
-	error = uri_create_https_str(&uri, head_data->uri,
+	error = uri_create_https_str_rrdp(&uri, head_data->uri,
 	    strlen(head_data->uri));
 	if (error)
 		return error;
@@ -1057,9 +1042,11 @@ release_uri:
  *
  * If the server didn't sent the file, due to the validation of
  * 'If-Modified-Since' header, return 0 and set @result to NULL.
+ *
+ * Set @force to true to omit 'If-Modified-Since' header.
  */
 int
-rrdp_parse_notification(struct rpki_uri *uri, bool log_operation,
+rrdp_parse_notification(struct rpki_uri *uri, bool log_operation, bool force,
     struct update_notification **result)
 {
 	long last_update;
@@ -1070,9 +1057,11 @@ rrdp_parse_notification(struct rpki_uri *uri, bool log_operation,
 
 	pr_val_debug("Processing notification '%s'.", uri_get_global(uri));
 	last_update = 0;
-	error = db_rrdp_uris_get_last_update(uri_get_global(uri), &last_update);
-	if (error && error != -ENOENT)
-		return error;
+	if (!force) {
+		error = db_rrdp_uris_get_last_update(uri_get_global(uri), &last_update);
+		if (error && error != -ENOENT)
+			return error;
+	}
 
 	error = download_file(uri, last_update, log_operation);
 	if (error < 0)
@@ -1123,7 +1112,7 @@ rrdp_parse_snapshot(struct update_notification *parent,
 	args.visited_uris = visited_uris;
 
 	pr_val_debug("Processing snapshot '%s'.", parent->snapshot.uri);
-	error = uri_create_https_str(&uri, parent->snapshot.uri,
+	error = uri_create_https_str_rrdp(&uri, parent->snapshot.uri,
 	    strlen(parent->snapshot.uri));
 	if (error)
 		return error;
