@@ -49,6 +49,12 @@ reqs_errors_log_summary(void)
 	/* Nothing here */
 }
 
+char const *
+db_rrdp_uris_workspace_get(void)
+{
+	return NULL;
+}
+
 char const *
 config_get_tal(void)
 {
@@ -301,3 +307,9 @@ config_set_http_priority(unsigned int value)
 {
 	http_priority = value;
 }
+
+unsigned int
+config_get_thread_pool_validation_max(void)
+{
+	return 10;
+}
