@@ -15,6 +15,7 @@
 #include "random.c"
 #include "crypto/base64.c"
 #include "rsync/rsync.c"
+#include "thread/thread_pool.c"
 
 /* Impersonate functions that won't be utilized by tests */
 
@@ -56,6 +57,17 @@ close_thread(pthread_t thread, char const *what)
 	/* Nothing to close */
 }
 
+int
+map_uri_to_local(char const *uri, char const *uri_prefix, char const *workspace,
+    char **result)
+{
+	/* These tests focus on global URIs, so set a dummy value */
+	*result = strdup("dummy");
+	if (*result == NULL)
+		return -ENOMEM;
+	return 0;
+}
+
 void
 fnstack_init(void)
 {
